/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity;

/**
 * Common constants of the identity solution.
 */
public class IdentityConstants {

	private IdentityConstants() {
	}

	public static final String SELF_ISSUED_ISSUER = "http://schemas.xmlsoap.org/ws/2005/05/identity/issuer/self";

	public static final String PREFIX = "ic";

	public static final String NS = "http://schemas.xmlsoap.org/ws/2005/05/identity";

	public static final String OPENID_NS = "http://schema.openid.net/2007/05";

	public final static String NS_MSFT_ADDR = "http://schemas.microsoft.com/ws/2005/05/addressing/none";

	public static final String IDENTITY_ADDRESSING_NS = "http://schemas.xmlsoap.org/ws/2006/02/addressingidentity";

	public final static String CLAIM_GIVEN_NAME = NS + "/claims/givenname";

	public final static String CLAIM_SURNAME = NS + "/claims/surname";

	public final static String CLAIM_EMAIL_ADDRESS = NS
			+ "/claims/emailaddress";

	public final static String CLAIM_STREET_ADDRESS = NS
			+ "/claims/streetaddress";

	public final static String CLAIM_CITY = NS + "/claims/locality";

	public final static String CLAIM_STATE = NS + "/claims/stateorprovince";

	public final static String CLAIM_POSTAL_CODE = NS + "/claims/postalcode";

	public final static String CLAIM_COUNTRY = NS + "/claims/country";

	public final static String CLAIM_HOME_PHONE = NS + "/claims/homephone";

	public final static String CLAIM_WORK_PHONE = NS + "/claims/otherphone";

	public final static String CLAIM_MOBILE_PHONE = NS + "/claims/mobile";

	public final static String CLAIM_PPID = NS
			+ "/claims/privatepersonalidentifier";

	public final static String CLAIM_NICKNAME = NS + "/claims/nickname";

	public final static String CLAIM_OPENID = OPENID_NS + "/claims/identifier";

	public final static String CLAIM_GENDER = NS + "/claims/gender";

	public final static String CLAIM_DOB = NS + "/claims/dateofbirth";

	public final static String CLAIM_TIME_ZONE = OPENID_NS + "/claims/timezone";

	public final static String CLAIM_LANGUAGE = OPENID_NS + "/claims/language";

	public final static String CLAIM_FULLNAME = OPENID_NS + "/claims/fullname";

	public final static String PARAM_SUPPORTED_TOKEN_TYPES = "SupportedTokenTypes";

	public final static String PARAM_CARD_NAME = "CardName";
	public final static String PARAM_VALUE_CARD_NAME = "WSO2 Managed Card";

	public final static String PARAM_VALID_PERIOD = "ValidPeriod";
	public final static String PARAM_VALUE_VALID_PERIOD = "365";

	public final static String SAML10_URL = "urn:oasis:names:tc:SAML:1.0:assertion";
	public final static String SAML11_URL = "http://docs.oasis-open.org/wss/oasis-wss-saml-token-profile-1.1#SAMLV1.1";
	public final static String SAML20_URL = "urn:oasis:names:tc:SAML:2.0:assertion";

	public final static String CARD_IMAGE_PATH = "/conf/card.jpg";

	public final static String PARAM_USE_SYMM_BINDING = "useSymmBinding";

	public final static String USER_VERIFICATION_PAGE = "/UserVerification.action";
	public final static String USER_VERIFICATION_PARAM = "confString";

	public final static String XML_TOKEN = "xmlToken";

	public final static String PROFILE_NAME = "profileName";

	public final static String DEFAULT_PROFILE = "Default Profile";

	public static class ErrorCodes {

		public final static String INVALID_OPENID = "invalidOpenID";
		public final static String INVALID_XMLTOKEN = "invalidXMLToken";
		public final static String OPENID_VERIFICATION_FAILED = "openIDVerificationFailed";
		public final static String OPENID_AUTHENTICATION_FAILED = "openIDAuthenticationFailed";
		public final static String INVALID_OPENID_AUTHENTICATION_REQUEST = "openIDInvalidAuthenticationRequest";
		public final static String RP_INITIATION_FAILED = "relyingPartyInitiationFailed";
		public final static String OPENID_TOKEN_EXTRACTION_FAILED = "openIDTokenExtractionFailed";
		public final static String REQUIRED_ATTRIBUTE_MISSING = "requredAttributeMissing";
		public final static String OPENID_DIRECT_RESP_FAILED = "openIDDirectResponseFailed";
		public final static String OPENID_RESP_GENERATION_FAILED = "openIDResponseGenerationFailed";
		public final static String OPENID_TOKEN_CREATION_FAILED = "openIDTokenCreationFailed";
		public final static String USERNAME_RETRIEVAL_FAILED = "usernameRetrievalFailed";
		public final static String NO_USERS_FOUND = "noUsersFound";
		public final static String CLAIM_RETRIEVAL_FAILED = "claimRetrievalFailed";
		public final static String NULL_OPEN_ID = "nullOpenID";
		public final static String INVALID_PPID = "invalidPPID";
		public final static String DB_CONNECTION_FAILURE = "dbConnectionFailure";
		public final static String NO_OPENID_FOUND = "noOpenIDFound";
		public final static String CARD_ISSUER_INIT_FAILURE = "cardIssuerInitializationFailure";
		public final static String PROFILE_RETRIEVAL_FAILURE = "profileRetrievalError";
		public final static String INVALID_USERNAME_FOR_OPENID = "invalidUserNameForOpenID";
		public final static String INVALID_OPENID_RETURNTO = "invalidOpenIDReturnTo";
	}

	/**
	 * Common constants related to OpenID.
	 */
	public static class OpenId {

		public final static String NS = "http://schema.openid.net";
		public final static String OPENID_URL = "http://specs.openid.net/auth/2.0";
		public final static String ATTR_MODE = "openid.mode";
		public final static String ATTR_IDENTITY = "openid.identity";
		public final static String ATTR_RESPONSE_NONCE = "openid.response_nonce";
		public final static String ATTR_OP_ENDPOINT = "openid.op_endpoint";
		public final static String ATTR_NS = "openid.ns";
		public final static String ATTR_CLAIM_ID = "openid.claimed_id";
		public final static String ATTR_RETURN_TO = "openid.return_to";
		public final static String ATTR_ASSOC_HANDLE = "openid.assoc_handle";
		public final static String ATTR_SIGNED = "openid.signed";
		public final static String ATTR_SIG = "openid.sig";
		public final static String OPENID_IDENTIFIER = "openid_identifier";
		public final static String ASSOCIATE = "associate";
		public final static String CHECKID_SETUP = "checkid_setup";
		public final static String CHECKID_IMMEDIATE = "checkid_immediate";
		public final static String CHECK_AUTHENTICATION = "check_authentication";
		public final static String DISC = "openid-disc";
		public static final String PREFIX = "openid";
		public final static String ASSERTION = "openidAssertion";
		public final static String COMPLETE = "complete";
		public final static String ACTION = "_action";
		public final static String OPENOD_RESPONSE = "id_res";
		public static final String AUTHENTICATED_AND_APPROVED = "authenticatedAndApproved";
		public final static String CANCEL = "cancel";
		public final static String PARAM_LIST = "parameterlist";

		public static class PapeAttributes {
			public final static String AUTH_POLICIES = "auth_policies";
			public final static String NIST_AUTH_LEVEL = "nist_auth_level";
			public final static String AUTH_AGE = "auth_age";
			public final static String PHISHING_RESISTANCE = "http://schemas.openid.net/pape/policies/2007/06/phishing-resistant";
			public final static String MULTI_FACTOR = "http://schemas.openid.net/pape/policies/2007/06/multi-factor";
			public final static String MULTI_FACTOR_PHYSICAL = "http://schemas.openid.net/pape/policies/2007/06/multi-factor-physical";
		}

		public static class SimpleRegAttributes {

			// As per the OpenID Simple Registration Extension 1.0 spec fields
			// below should be included in the Identity Provider's
			// response when "openid.mode" is "id_res"

			public final static String NS_SREG = "http://openid.net/sreg/1.0";
			public final static String NS_SREG_1 = "http://openid.net/extensions/sreg/1.1";
			public final static String SREG = "openid.sreg.";
			public final static String OP_SREG = "openid.ns.sreg";

			public final static String NICK_NAME = "nickname";
			public final static String EMAIL = "email";
			public final static String FULL_NAME = "fullname";
			public final static String DOB = "dob";
			public final static String GENDER = "gender";
			public final static String POSTAL_CODE = "postcode";
			public final static String COUNTRY = "country";
			public final static String LANGUAGE = "language";
			public final static String TIMEZONE = "timezone";

			public final static String FULL_NAME_NS = "http://schema.openid.net/2007/05/claims/fullname";
			public final static String DOB_NS = "http://schema.openid.net/2007/05/claims/dob";
			public final static String GENDER_NS = "http://schema.openid.net/2007/05/claims/gender";
			public final static String LANGUAGE_NS = "http://schema.openid.net/2007/05/claims/language";
			public final static String TIMEZONE_NS = "http://schema.openid.net/2007/05/claims/timezone";

		}

		public static class ExchangeAttributes extends SimpleRegAttributes {

			public final static String NS = "http://axschema.org";
			public final static String NS_AX = "http://openid.net/srv/ax/1.0";
			public final static String EXT = "openid.ns.ext1";
			public final static String MODE = "openid.ext1.mode";
			public final static String TYPE = "openid.ext1.type.";
			public final static String VALUE = "openid.ext1.value.";
			public final static String FETCH_RESPONSE = "fetch_response";

			public final static String NICK_NAME_NS = NS
					+ "/namePerson/friendly";
			public final static String EMAIL_NS = NS + "/contact/email";
			public final static String FULL_NAME_NS = NS + "/namePerson";
			public final static String DOB_NS = NS + "/birthDate";
			public final static String GENDER_NS = NS + "/person/gender";
			public final static String POSTAL_CODE_NS = NS
					+ "/contact/postalCode/home";
			public final static String COUNTRY_NS = NS
					+ "/contact/country/home";
			public final static String LANGUAGE_NS = NS + "/pref/language";
			public final static String TIMEZONE_NS = NS + "/pref/timezone";

		}
	}
}
