/*
 * Copyright 2005,2006 WSO2, Inc. http://www.wso2.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.solutions.identity.admin;

import org.apache.axis2.AxisFault;
import org.wso2.solutions.identity.IdentityConstants;
import org.wso2.solutions.identity.IdentityProviderException;
import org.wso2.solutions.identity.persistence.IPPersistenceManager;
import org.wso2.solutions.identity.persistence.dataobject.ClaimDO;
import org.wso2.solutions.identity.persistence.dataobject.DialectDO;

public class ClaimsAdmin {

    IPPersistenceManager dbman;

    public ClaimsAdmin() throws IdentityProviderException {
        dbman = IPPersistenceManager.getPersistanceManager();
    }

    // CRUDClaims
    public void createClaim(ClaimDO claim) throws AxisFault {
        try {
            dbman.create(claim);
        } catch (IdentityProviderException e) {
            throw new AxisFault("IdentityPersistanceException", e);
        }
    }

    public void createClaim(String dialectUri, String uri, String displayTag,
            String description) throws IdentityProviderException {

        DialectDO dialect = this.getDialect(dialectUri);

        ClaimDO claimDO = new ClaimDO();
        claimDO.setUri(uri);
        claimDO.setDisplayTag(displayTag);
        claimDO.setDescription(description);
        claimDO.setDialect(dialect);

        dbman.create(claimDO);

    }

    public void createClaim(String dialectUri, String uri, String displayTag,
            String description, String openIDTag)
            throws IdentityProviderException {

        DialectDO dialect = this.getDialect(dialectUri);

        ClaimDO claimDO = new ClaimDO();
        claimDO.setUri(uri);
        claimDO.setDisplayTag(displayTag);
        claimDO.setDescription(description);
        claimDO.setDialect(dialect);

        if (openIDTag != null && openIDTag.trim().length() > 0)
            claimDO.setOpenIDTag(openIDTag);

        dbman.create(claimDO);

    }

    public void createDialect(String dialectUri, String dialectInfo)
            throws IdentityProviderException {
        DialectDO dialectDO = new DialectDO();
        dialectDO.setDialectUri(dialectUri);
        dialectDO.setDialectInfo(dialectInfo);
        dbman.create(dialectDO);
    }

    public ClaimDO[] getAllClaims() {
        return dbman.getAllSupportedClaims();
    }

    public void updateClaim(ClaimDO claim) throws AxisFault {
        dbman.update(claim);
    }

    public void deleteClaim(ClaimDO claim) {
        dbman.delete(claim);
    }

    public ClaimDO findClaimByURI(String value) {
        return dbman.getClaimByURI(value);
    }

    public ClaimDO findClaim(Long id) {
        return (ClaimDO) dbman.getDataObject("ClaimDO", id);
    }

    public DialectDO[] getAllDialects() {
        return dbman.getAllSupportedDialects();
    }

    public DialectDO getDialect(String uri) {
        return dbman.getDialect(uri);
    }

    public ClaimDO[] getAllSupportedClaims() {
        return dbman.getAllSupportedClaims();
    }

    public ClaimDO[] getAllMappedClaims() {
        return dbman.getAllMappedClaims();
    }

    public ClaimDO[] getAllEnabledClaims() {
        return dbman.getAllEnabledClaims();
    }

    public ClaimDO[] getAllMappedEnabledClaims() {
        return dbman.getAllMappedEnabledClaims();
    }
    
    public String getMappedOpenIDTag(String uri)
    {
        return dbman.getMappedOpenIDTag(uri);
    }
    
    public boolean isValidOpenIDTag(String openIDTag)
    {
        return dbman.isValidOpenIDTag(openIDTag);
    }

    public void resetClaimMappings() {
        ClaimDO[] claimDOs = this.getAllMappedClaims();
        for (int i = 0; i < claimDOs.length; i++) {
            ClaimDO claim = claimDOs[i];
            claim.setAttrId(ClaimDO.DAFAULT_ATTR_ID);

            //Disable all claims other that ppid
            //when mappings are reset
            if (!claim.getUri().equals(IdentityConstants.CLAIM_PPID)) {
                claim.setSupported(false);
            }

            dbman.update(claim);
        }
    }
}
