/*
 * Copyright 2005,2006 WSO2, Inc. http://www.wso2.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.solutions.identity.admin;

import java.util.Date;

import org.wso2.solutions.identity.IdentityProviderException;
import org.wso2.solutions.identity.persistence.IPPersistenceManager;
import org.wso2.solutions.identity.persistence.dataobject.InfoCardDO;
import org.wso2.solutions.identity.persistence.dataobject.IssuedTokensDO;
import org.wso2.solutions.identity.persistence.dataobject.RevokedInfoCardDO;

public class InfoCardTokenAdmin {

    private IPPersistenceManager dbman;

    public InfoCardTokenAdmin() throws IdentityProviderException {
        dbman = IPPersistenceManager.getPersistanceManager();
    }

    public InfoCardDO[] getCardIdsForUser(String username) {
        return dbman.getCardIdsForUser(username);
    }

    public int deleteCardsofUser(String username) {
        String stmt = "delete InfoCardDO where userId = :name";
        return dbman.executeSingleDMLStatement(stmt, "name", username);
    }

    /**
     * Obtain all the issued information cards.
     * @return An array of <code>org.wso2.solutions.identity.persistence.dataobject.InfoCardDO</code> instances. 
     */
    public InfoCardDO[] getAllInfoCards() {
        return dbman.getAllInfoCards();
    }

    public IssuedTokensDO[] getTokensIssuedBetween(String username, Date startDate, Date endDate) {
        return dbman.getAllTokensOfUser(username);
    }
    
    public RevokedInfoCardDO[] getRevokedInfoCards() {
        return dbman.getAllRevokedInfoCards();
    }

    public void revokeInformationCard(String cardId)
            throws IdentityProviderException {
        dbman.revokeInfoCard(dbman.getInfoCard(cardId));
    }
    
    public int getInfoCardCount(){
        return dbman.getCount("InfoCardDO");
    }
    
    public int getTokenRequestCount(){
        return dbman.getCount("IssuedTokensDO");
    }

}
