/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.cards.model;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.om.util.Base64;
import org.apache.ws.security.WSConstants;
import org.apache.xml.security.utils.Constants;
import org.wso2.solutions.identity.IdentityConstants;

import javax.xml.namespace.QName;

import java.security.cert.CertificateEncodingException;
import java.security.cert.X509Certificate;

public class Identity implements CardElement {

    public final static QName NAME = new QName(
            "http://schemas.xmlsoap.org/ws/2006/02/addressingidentity",
            "Identity", IdentityConstants.PREFIX);

    private X509Certificate certificate;

    public OMElement serialize() throws CardModelException {

        if (this.certificate == null) {
            throw new CardModelException(CardModelException.DEFAULT,
                    new String[] { "Certificate missing" });
        }

        OMFactory fac = OMAbstractFactory.getOMFactory();
        OMElement elem = fac.createOMElement(NAME);

        OMElement kiElem = fac.createOMElement(new QName(WSConstants.SIG_NS,
                Constants._TAG_KEYINFO, WSConstants.SIG_PREFIX), elem);
        OMElement x509DataElem = fac.createOMElement(new QName(
                WSConstants.SIG_NS, Constants._TAG_X509DATA,
                WSConstants.SIG_PREFIX), kiElem);
        OMElement certElem = fac.createOMElement(new QName(WSConstants.SIG_NS,
                Constants._TAG_X509CERTIFICATE, WSConstants.SIG_PREFIX),
                x509DataElem);
        try {
            certElem.setText(Base64.encode(this.certificate.getEncoded()));
        } catch (CertificateEncodingException e) {
            throw new CardModelException(CardModelException.DEFAULT, e);
        }
        return elem;
    }

    public X509Certificate getCertificate() {
        return certificate;
    }

    public void setCertificate(X509Certificate certificate) {
        this.certificate = certificate;
    }

}
