/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.cards.model;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.wso2.solutions.identity.IdentityConstants;

import javax.xml.namespace.QName;

public class InformationCardReference implements CardElement {

    public final static QName NAME = new QName(IdentityConstants.NS,
            "InformationCardReference", IdentityConstants.PREFIX);

    public final static QName CARD_ID = new QName(IdentityConstants.NS,
            "CardId", IdentityConstants.PREFIX);

    public final static QName CARD_VERSION = new QName(IdentityConstants.NS,
            "CardVersion", IdentityConstants.PREFIX);

    private String cardId = null;

    private int cardVersion = 1;

    public InformationCardReference(String cardId, int cardVersion) {
        this.cardId = cardId;
        this.cardVersion = cardVersion;
    }

    public String getCardId() {
        return cardId;
    }

    public int getCardVersion() {
        return cardVersion;
    }

    public OMElement serialize() throws CardModelException {
        OMFactory fac = OMAbstractFactory.getOMFactory();
        OMElement elem = fac.createOMElement(NAME);

        OMElement cardIdElem = fac.createOMElement(CARD_ID, elem);
        cardIdElem.setText(this.cardId);

        OMElement cardVersionElem = fac.createOMElement(CARD_VERSION, elem);
        cardVersionElem.setText(Integer.toString(this.cardVersion));

        return elem;
    }

}
