/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.cards.model;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.wso2.solutions.identity.IdentityConstants;

import javax.xml.namespace.QName;

public class SelfIssuedCredential implements Credential {

    public final static QName NAME = new QName(IdentityConstants.NS,
            "SelfIssuedCredential", IdentityConstants.PREFIX);

    private QName PPID = new QName(IdentityConstants.NS,
            "PrivatePersonalIdentifier", IdentityConstants.PREFIX);

    private String personalPrivateIdentifier = null;

    /**
     * Create a new SelfIssuedCredential with a personal private identifier.
     * 
     * @param ppid
     *            Base64 encoded personal private identifier value.
     */
    public SelfIssuedCredential(String ppid) {
        this.personalPrivateIdentifier = ppid;
    }

    public String getPersonalPrivateIdentifier() {
        return personalPrivateIdentifier;
    }

    public OMElement serialize() throws CardModelException {

        if (this.personalPrivateIdentifier == null) {
            throw new CardModelException(
                    CardModelException.REQUIRED_ELEMENT_MISSING,
                    new String[] { PPID.toString() });
        }

        OMFactory fac = OMAbstractFactory.getOMFactory();
        OMElement elem = fac.createOMElement(NAME);

        OMElement ppidElem = fac.createOMElement(PPID, elem);
        ppidElem.setText(this.personalPrivateIdentifier);

        return elem;
    }

}
