/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.cards.model;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.wso2.solutions.identity.IdentityConstants;

import javax.xml.namespace.QName;

public class SupportedClaimType implements CardElement {

    public final static QName NAME = new QName(IdentityConstants.NS,
            "SupportedClaimType", IdentityConstants.PREFIX);

    private QName DISPLAY_TAG = new QName(IdentityConstants.NS, "DisplayTag",
            IdentityConstants.PREFIX);

    private QName DESCRIPTION = new QName(IdentityConstants.NS, "Description",
            IdentityConstants.PREFIX);

    private String ATTR_URI = "Uri";

    private String uri;

    private String displayTag;

    private String description;

    public SupportedClaimType(String uri) {
        this.uri = uri;
    }

    public OMElement serialize() throws CardModelException {

        if (this.uri == null) {
            throw new CardModelException(
                    CardModelException.REQUIRED_ATTRIBUTE_MISSING,
                    new String[] { ATTR_URI });
        }

        OMFactory fac = OMAbstractFactory.getOMFactory();
        OMElement elem = fac.createOMElement(NAME);

        elem.addAttribute(ATTR_URI, this.uri, null);

        if (displayTag != null) {
            OMElement dispTagElement = fac.createOMElement(DISPLAY_TAG, elem);
            dispTagElement.setText(this.displayTag);
        }

        if (description != null) {
            OMElement descElement = fac.createOMElement(DESCRIPTION, elem);
            descElement.setText(this.description);
        }

        return elem;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public String getDisplayTag() {
        return displayTag;
    }

    public void setDisplayTag(String displayTag) {
        this.displayTag = displayTag;
    }

    public String getUri() {
        return uri;
    }

    public void setUri(String uri) {
        this.uri = uri;
    }

}
