/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.cards.model;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.wso2.solutions.identity.IdentityConstants;

import javax.xml.namespace.QName;

import java.util.ArrayList;
import java.util.Iterator;

public class SupportedClaimTypeList implements CardElement {

    public final static QName NAME = new QName(IdentityConstants.NS,
            "SupportedClaimTypeList", IdentityConstants.PREFIX);

    ArrayList claimTypeList = new ArrayList();

    public OMElement serialize() throws CardModelException {
        OMFactory fac = OMAbstractFactory.getOMFactory();
        OMElement elem = fac.createOMElement(NAME);

        if (claimTypeList.isEmpty()) {
            throw new CardModelException(
                    CardModelException.REQUIRED_ELEMENT_MISSING,
                    new String[] { SupportedClaimType.NAME.toString() });
        }

        Iterator claimTypeIterator = claimTypeList.iterator();
        while (claimTypeIterator.hasNext()) {
            SupportedClaimType claimType = (SupportedClaimType) claimTypeIterator
                    .next();
            elem.addChild(claimType.serialize());
        }

        return elem;
    }

    public void addSupportedClaimType(SupportedClaimType claimType) {
        this.claimTypeList.add(claimType);
    }

}
