/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.cards.model;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.rahas.RahasConstants;
import org.apache.rahas.TrustException;
import org.apache.rahas.TrustUtil;
import org.wso2.solutions.identity.IdentityConstants;

import javax.xml.namespace.QName;

import java.util.ArrayList;
import java.util.Iterator;

public class SupportedTokenTypeList implements CardElement {

    public final static QName NAME = new QName(IdentityConstants.NS,
            "SupportedTokenTypeList", IdentityConstants.PREFIX);

    private ArrayList tokenTypeList = new ArrayList();

    public void addSupportedTokenType(String tokenType) {
        this.tokenTypeList.add(tokenType);
    }

    public OMElement serialize() throws CardModelException {
        OMFactory fac = OMAbstractFactory.getOMFactory();
        OMElement elem = fac.createOMElement(NAME);

        if (tokenTypeList.isEmpty()) {
            throw new CardModelException(
                    CardModelException.REQUIRED_ELEMENT_MISSING,
                    // TODO Trust version ???
                    new String[] { (new QName(RahasConstants.WST_NS_05_02,
                            RahasConstants.LocalNames.TOKEN_TYPE).toString()) });

        }

        try {
            Iterator tokenTypeIterator = tokenTypeList.iterator();
            while (tokenTypeIterator.hasNext()) {
                String tokenType = (String) tokenTypeIterator.next();
                OMElement tokenTypeElem = TrustUtil.createTokenTypeElement(
                        RahasConstants.VERSION_05_02, elem);
                tokenTypeElem.setText(tokenType);
            }
        } catch (TrustException e) {
            throw new CardModelException(CardModelException.DEFAULT, e);
        }
        return elem;
    }
}
