/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.cards.model;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.wso2.solutions.identity.IdentityConstants;

import javax.xml.namespace.QName;

public class UserCredential implements CardElement {

    public final static QName NAME = new QName(IdentityConstants.NS,
            "UserCredential", IdentityConstants.PREFIX);

    public final static QName DISPLAY_CRED_HINT = new QName(
            IdentityConstants.NS, "DisplayCredentialHint",
            IdentityConstants.PREFIX);

    private String displayCredentialHint = null;

    private Credential credential;

    public UserCredential(Credential credential) {
        this.credential = credential;
    }

    public OMElement serialize() throws CardModelException {

        if (credential == null) {
            throw new CardModelException(
                    CardModelException.REQUIRED_ELEMENT_MISSING,
                    new String[] { UsernamePasswordCredential.NAME.toString()
                            + "|" + SelfIssuedCredential.NAME.toString() + "|"
                            + KerberosV5Credential.NAME.toString() + "|"
                            + X509V3Credential.NAME.toString() });
        }

        OMFactory fac = OMAbstractFactory.getOMFactory();
        OMElement elem = fac.createOMElement(NAME);

        if (this.displayCredentialHint != null) {
            OMElement dispCredHintElem = fac.createOMElement(DISPLAY_CRED_HINT,
                    elem);
            dispCredHintElem.setText(this.displayCredentialHint);
        }

        elem.addChild(credential.serialize());

        return elem;
    }

    public String getDisplayCredentialHint() {
        return displayCredentialHint;
    }

    public void setDisplayCredentialHint(String displayCredentialHint) {
        this.displayCredentialHint = displayCredentialHint;
    }

    public Credential getCredential() {
        return credential;
    }

    public String getCredentialId() throws CardModelException {

        if (this.credential instanceof UsernamePasswordCredential) {
            return ((UsernamePasswordCredential) this.credential).getUsername();
        } else if (this.credential instanceof SelfIssuedCredential) {
            return ((SelfIssuedCredential) this.credential)
                    .getPersonalPrivateIdentifier();
        } else {
            throw new CardModelException("unsupportedCredentialType",
                    new String[] { this.credential.getClass().getName() });
        }
    }

}
