/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.cards.model;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.ws.security.WSConstants;
import org.apache.ws.security.message.token.BinarySecurity;
import org.apache.ws.security.message.token.SecurityTokenReference;
import org.apache.xml.security.utils.Constants;
import org.wso2.solutions.identity.IdentityConstants;

import javax.xml.namespace.QName;

public class X509V3Credential implements Credential {

    public final static QName NAME = new QName(IdentityConstants.NS,
            "X509V3Credential", IdentityConstants.PREFIX);

    private String base64EncodedThumbprintValue = null;

    /**
     * {@inheritDoc}
     */
    public OMElement serialize() throws CardModelException {
        
        /*
         *
          <ic:X509V3Credential>
            <ds:X509Data>
              <wsse:KeyIdentifier
                ValueType="http://docs.oasis-open.org/wss/2004/xx/oasis-2004xx-wss-
        soap-message-security-1.1#ThumbprintSHA1"
                EncodingType="http://docs.oasis-open.org/wss/2004/01/oasis200401-wss-
        soap-message-security-1.0#Base64Binary">
                xs:base64binary
              </wsse:KeyIdentifier>
            </ds:X509Data>
          </ic:X509V3Credential>
         */
        
        OMFactory fac = OMAbstractFactory.getOMFactory();
        OMElement elem = fac.createOMElement(NAME);

        OMElement x509DataElem = fac.createOMElement(new QName(
                WSConstants.SIG_NS, Constants._TAG_X509DATA, WSConstants.SIG_PREFIX), elem);
        OMElement keyIdElem = fac.createOMElement(new QName(
                WSConstants.WSSE_NS, "KeyIdentifier", WSConstants.WSSE_PREFIX),
                x509DataElem);
        keyIdElem.addAttribute("ValueType", "http://docs.oasis-open.org/wss/2004/xx/oasis-2004xx-wss-soap-message-security-1.1#ThumbprintSHA1", null);
//        keyIdElem.addAttribute("EncodingType", BinarySecurity.BASE64_ENCODING, null);
        keyIdElem.setText(this.base64EncodedThumbprintValue);
        
        return elem;
    }

    
    public void setBase64EncodedThumbprintValue(String base64EncodedThumbprintValue) {
        this.base64EncodedThumbprintValue = base64EncodedThumbprintValue;
    }
}
