/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.persistence.dao;

import java.util.List;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.wso2.solutions.identity.persistence.HibernateConfig;
import org.wso2.solutions.identity.persistence.dataobject.RealmConfigurationDO;

public class RealmConfigurationDAO extends BaseDAO {

    public RealmConfigurationDAO(HibernateConfig config) {
        super(config);
    }

    public RealmConfigurationDO[] getAllRealmConfigurations() {
        Session session = hbConfig.getCurrentSession();
        Transaction tx = session.beginTransaction();
        String stmt = "from RealmConfigurationDO";

        RealmConfigurationDO[] realmConfigs = new RealmConfigurationDO[0];
        try {
            Query query = session.createQuery(stmt);
            List lst = query.list();
            realmConfigs = (RealmConfigurationDO[]) lst
                    .toArray(new RealmConfigurationDO[lst.size()]);
            tx.commit();
        } catch (Throwable e) {
            tx.rollback();
            String msg = messages
                    .getMessage("errorQuerryingRealmConfiguration");
            log.error(msg, e);
            throw new RuntimeException(msg, e);
        } finally {
            hbConfig.closeSession();
        }

        return realmConfigs;
    }

    /**
     * Provides the selected realm for the identity provider.
     * 
     * @return Returns the selected realm for the identity provider.
     */
    public RealmConfigurationDO getEffectiveRealmConfiguration() {
        Session session = hbConfig.getCurrentSession();
        Transaction tx = session.beginTransaction();
        String stmt = "from RealmConfigurationDO as rc where rc.effective = 1";

        RealmConfigurationDO realmConfig = null;
        try {
            realmConfig = (RealmConfigurationDO) session.createQuery(stmt)
                    .uniqueResult();
            tx.commit();
        } catch (Throwable e) {
            tx.rollback();
            String msg = messages
                    .getMessage("errorQuerryingRealmConfiguration");
            log.error(msg, e);
            throw new RuntimeException(msg, e);
        } finally {
            hbConfig.closeSession();
        }

        return realmConfig;
    }

    /**
     * Returns the realm configuration identified by the given name.
     * 
     * @param name
     *            Name of the required realm configuration.
     * @return Returns the realm configuration identified by the given name.
     */
    public RealmConfigurationDO getRealmConfiguration(String name) {
        Session session = hbConfig.getCurrentSession();
        Transaction tx = session.beginTransaction();
        String stmt = "from RealmConfigurationDO as rc where rc.name = '"
                + name + "'";

        RealmConfigurationDO realmConfig = null;
        try {
            realmConfig = (RealmConfigurationDO) session.createQuery(stmt)
                    .uniqueResult();
            tx.commit();
        } catch (Throwable e) {
            tx.rollback();
            String msg = messages
                    .getMessage("errorQuerryingRealmConfiguration");
            log.error(msg, e);
            throw new RuntimeException(msg, e);
        } finally {
            hbConfig.closeSession();
        }

        return realmConfig;
    }

}
