/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.solutions.identity.persistence.dao;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Expression;
import org.wso2.solutions.identity.persistence.HibernateConfig;
import org.wso2.solutions.identity.persistence.dataobject.RegisteredInfoCardInfoDO;

import java.util.List;

public class RegisteredInfoCardInfoDAO extends BaseDAO {

    public RegisteredInfoCardInfoDAO(HibernateConfig config) {
        super(config);
    }

    public RegisteredInfoCardInfoDO[] getAllInfoForUser(String userId) {
        Session session = hbConfig.getCurrentSession();
        Transaction tx = session.beginTransaction();
        try {
            Criteria criteria = session
                    .createCriteria(RegisteredInfoCardInfoDO.class);
            criteria.add(Expression.eq("userId", userId));
            List lst = criteria.list();
            return (RegisteredInfoCardInfoDO[]) lst
                    .toArray(new RegisteredInfoCardInfoDO[lst.size()]);
        } catch (Throwable e) {
            tx.rollback();
            String msg = messages
                    .getMessage("errorQuerryingRegisteredInfoCardInfo");
            log.error(msg, e);
            throw new RuntimeException(msg, e);
        } finally {
            hbConfig.closeSession();
        }
    }
    
     public RegisteredInfoCardInfoDO getInformationCardInfo(String ppid) {
        Session session = hbConfig.getCurrentSession();
        Transaction tx = session.beginTransaction();
        try {
            Criteria criteria = session
                    .createCriteria(RegisteredInfoCardInfoDO.class);
            criteria.add(Expression.eq("ppid", ppid));
            return (RegisteredInfoCardInfoDO) criteria.uniqueResult();
        } catch (Throwable e) {
            tx.rollback();
            String msg = messages
                    .getMessage("errorQuerryingRegisteredInfoCardInfo");
            log.error(msg, e);
            throw new RuntimeException(msg, e);
        } finally {
            hbConfig.closeSession();
        }
    }
    
    /**
     * This method extracts the primary username if there is one.
     * If there is no primary username ... null will be returned. 
     */
    public String extractPrimaryUserName(String ppid){
        String username = null;
        Session session = hbConfig.getCurrentSession();
        Transaction tx = session.beginTransaction();
        try {
            Criteria criteria = session
                    .createCriteria(RegisteredInfoCardInfoDO.class);
            criteria.add(Expression.eq("ppid", ppid));
            RegisteredInfoCardInfoDO ido = (RegisteredInfoCardInfoDO)criteria.uniqueResult();
            if(ido != null){
                username = ido.getUserId();
            }
        } catch (Throwable e) {
            tx.rollback();
            String msg = messages
                    .getMessage("errorQuerryingRegisteredInfoCardInfo");
            log.error(msg, e);
            throw new RuntimeException(msg, e);
        } finally {
            hbConfig.closeSession();
        }
        
        return username;        
    }

}
