/*                                                                             
 * Copyright 2005,2006 WSO2, Inc. http://www.wso2.org
 *                                                                             
 * Licensed under the Apache License, Version 2.0 (the "License");             
 * you may not use this file except in compliance with the License.            
 * You may obtain a copy of the License at                                     
 *                                                                             
 *      http://www.apache.org/licenses/LICENSE-2.0                             
 *                                                                             
 * Unless required by applicable law or agreed to in writing, software         
 * distributed under the License is distributed on an "AS IS" BASIS,           
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.    
 * See the License for the specific language governing permissions and         
 * limitations under the License.                                              
 */
package org.wso2.solutions.identity.persistence.dao;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.wso2.solutions.identity.persistence.HibernateConfig;
import org.wso2.solutions.identity.persistence.dataobject.PPIDValueDO;
import org.wso2.solutions.identity.persistence.dataobject.RelyingPartyDO;

import java.util.List;
import java.util.Set;

public class RelyingPartyDAO extends BaseDAO {

    protected Log log = LogFactory.getLog(RelyingPartyDAO.class);

    public RelyingPartyDAO(HibernateConfig config) {
        super(config);
    }

    public RelyingPartyDO getRelyingPartyDO(String url) {
        Session session = hbConfig.getCurrentSession();
        Transaction tx = session.beginTransaction();

        RelyingPartyDO rp = null;
        try {

            rp = getRelyingPartyDO(url, session);
            tx.commit();

        } catch (Throwable e) {
            tx.rollback();
            String msg = messages.getMessage("errorQuerryingRelyingParty");
            log.error(msg, e);
            throw new RuntimeException(msg, e);
        } finally {
            hbConfig.closeSession();
        }

        return rp;
    }

    public RelyingPartyDO[] getAllRelyingParties() {
        String stmt = "from RelyingPartyDO";
        Session session = hbConfig.getCurrentSession();
        Transaction tx = session.beginTransaction();

        RelyingPartyDO[] rps = null;
        try {

            Query query = session.createQuery(stmt);
            List lst = query.list();
            rps = (RelyingPartyDO[]) lst
                    .toArray(new RelyingPartyDO[lst.size()]);
            tx.commit();
        } catch (Throwable e) {
            tx.rollback();
            String msg = messages.getMessage("errorQuerryingRelyingParty");
            log.error(msg, e);
            throw new RuntimeException(msg, e);
        } finally {
            hbConfig.closeSession();
        }

        return rps;

    }

    public PPIDValueDO[] getPPIDValues(String url) {
        Session session = hbConfig.getCurrentSession();
        Transaction tx = session.beginTransaction();

        PPIDValueDO[] ppidVals = null;
        try {

            RelyingPartyDO rp = getRelyingPartyDO(url, session);
            Set setppid = rp.getPpidValues();
            ppidVals = (PPIDValueDO[]) setppid.toArray(new PPIDValueDO[setppid
                    .size()]);
            tx.commit();

        } catch (Throwable e) {
            tx.rollback();
            String msg = messages.getMessage("errorQuerryingRelyingParty");
            log.error(msg, e);
            throw new RuntimeException(msg, e);
        } finally {
            hbConfig.closeSession();
        }

        return ppidVals;
    }

    public RelyingPartyDO getRelyingPartyDO(String rpURL, Session session) {
        String query = "from RelyingPartyDO as rp where rp.hostName='"
                + rpURL.trim() + "'";
        RelyingPartyDO rp = (RelyingPartyDO) session.createQuery(query)
                .uniqueResult();
        return rp;
    }
}
