/*                                                                             
 * Copyright 2005,2006 WSO2, Inc. http://www.wso2.org
 *                                                                             
 * Licensed under the Apache License, Version 2.0 (the "License");             
 * you may not use this file except in compliance with the License.            
 * You may obtain a copy of the License at                                     
 *                                                                             
 *      http://www.apache.org/licenses/LICENSE-2.0                             
 *                                                                             
 * Unless required by applicable law or agreed to in writing, software         
 * distributed under the License is distributed on an "AS IS" BASIS,           
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.    
 * See the License for the specific language governing permissions and         
 * limitations under the License.                                              
 */
package org.wso2.solutions.identity.sts;

import org.apache.axis2.AxisFault;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.security.WSPasswordCallback;
import org.wso2.solutions.identity.IdentityProviderConstants;
import org.wso2.solutions.identity.IdentityProviderException;
import org.wso2.solutions.identity.UserStore;
import org.wso2.solutions.identity.i18n.Messages;

import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.UnsupportedCallbackException;

import java.io.IOException;

public class IPPasswordCallback implements CallbackHandler {

    private static Log log = LogFactory.getLog(IPPasswordCallback.class);
    private static Messages messages = Messages.getInstance(IdentityProviderConstants.RESOURCES);
    
    private CallbackHandler wsasCallbackHandler;
    
    /**
     * @param value
     */
    public IPPasswordCallback(CallbackHandler wsasCallbackHandler) {
        this.wsasCallbackHandler = wsasCallbackHandler;
    }

    public void handle(Callback[] callbacks) throws IOException,
            UnsupportedCallbackException {

        try {
            for (int i = 0; i < callbacks.length; i++) {
                WSPasswordCallback pwcb = (WSPasswordCallback) callbacks[i];
                if (pwcb.getUsage() == WSPasswordCallback.USERNAME_TOKEN_UNKNOWN) {
                    String username = pwcb.getIdentifer();
                    String passwd = pwcb.getPassword();
                    if (passwd != null) {
                        UserStore connector = UserStore.getInstance();

                        if (connector.authenticate(username, passwd) == true) {
                            continue;
                        } else {
                            throw new IOException("User not authenticated");
                        }

                    } else {
                        throw new IOException("User not authenticated");
                    }
                } else {
                    wsasCallbackHandler.handle(new Callback[] {callbacks[i]});
                }
            }
        } catch (IdentityProviderException e) {
            log.error(messages.getMessage("errorInPasswordCallbackHandler"), e);
        }
    }

}